#include "B4RDefines.h"
namespace B4R {
	void B4RPID::Initialize(Double Input, Double Output, Double Setpoint, Double Kp, Double Ki, Double Kd, Byte POn, Byte ControllerDirection) {
		rpid = new (backend) PID(Input, Output, Setpoint, Kp, Ki, Kd, POn, ControllerDirection);
	
		myOutput = Output;
		myInput = Input;
		mySetpoint = Setpoint;
		inAuto = false;

		rpid->SetOutputLimits(0, 255);				//default output limit corresponds to
													//the arduino pwm limits

		SampleTime = 100;							//default Controller Sample Time is 0.1 seconds

		rpid->SetControllerDirection(ControllerDirection);
		rpid->SetTunings(Kp, Ki, Kd, POn);

		lastTime = millis()-SampleTime;

	}
	
//	void B4Rpid::Initialize(PID(Double* Input, Double* Output, Double* Setpoint, Double Kp, Double Ki, Double Kd, int ControllerDirection)) {
//		rpid = new (bePid) PID(Input, Output, Setpoint, Kp, Ki, Kd, P_ON_E, ControllerDirection);}
//		rpid->init();  ****************

	
		// * sets PID to either Manual (0) or Auto (non-0)
		void B4Rpid::SetMode(Byte Mode)
		{
			bool newAuto = (Mode == AUTOMATIC);
			if(newAuto && !inAuto)
			{  /*we just went from manual to auto*/
				outputSum = myOutput;
				lastInput = myInput;
				if(outputSum > outMax) outputSum = outMax;
				else if(outputSum < outMin) outputSum = outMin;
			}
			inAuto = newAuto;
		}
		
		// * performs the PID calculation. it should be called every time loop() cycles. ON/OFF and
        //   calculation frequency can be set using SetMode - SetSampleTime respectively
		bool B4Rpid::Compute(void)
		{
			if(!inAuto) return false;
			ULong now = millis();
			ULong timeChange = (now - lastTime);
			if(timeChange>=SampleTime)
			{
			/*Compute all the working error variables*/
			Double input = myInput;
			Double error = mySetpoint - input;
			Double dInput = (input - lastInput);
			outputSum+= (ki * error);

			/*Add Proportional on Measurement, if P_ON_M is specified*/
			if(!pOnE) outputSum-= kp * dInput;

			if(outputSum > outMax) outputSum= outMax;
			else if(outputSum < outMin) outputSum= outMin;

			/*Add Proportional on Error, if P_ON_E is specified*/
			Double output;
			if(pOnE) output = kp * error;
			else output = 0;

			/*Compute Rest of PID Output*/
			output += outputSum - kd * dInput;

			if(output > outMax) output = outMax;
			else if(output < outMin) output = outMin;
			myOutput = output;

			/*Remember some variables for next time*/
			lastInput = input;
			lastTime = now;
			return true;
			}
			else return false;
		}
		
		void B4Rpid::SetOutputLimits(Double Min, Double Max)
		{
			if(Min >= Max) return;
			outMin = Min;
			outMax = Max;
			
			if (inAuto) 
			{
				if(myOutput > outMax) myOutput = outMax;
				else if(myOutput < outMin) myOutput = outMin;

				if(outputSum > outMax) outputSum= outMax;
				else if(outputSum < outMin) outputSum= outMin;
			}
		}
		
		void B4Rpid::SetTunings(Double Kp, Double Ki, Double Kd){
	         rpid->SetTunings(Kp, Ki, Kd);
		}
	
		void B4Rpid::SetTunings(Double Kp, Double Ki, Double Kd, Byte POn){
	         rpid->SetTunings(Kp, Ki, Kd, POn);
		}
	
		void B4Rpid::SetControllerDirection(Byte Direction){
			rpid->SetControllerDirection(Direction);
		}
		
		void B4Rpid::SetSampleTime(Int NewSampleTime){
			rpid->SetSampleTime(NewSampleTime);
		}
		
		Double B4Rpid::GetKp(void) const { return rpid->GetKp();}
		Double B4Rpid::GetKi(void) const { return rpid->GetKi();}
		Double B4Rpid::GetKd(void) const { return rpid->GetKd();}

		Int 	B4Rpid::GetMode(void) const { return rpid->GetMode();}
		Int 	B4Rpid::GetDirection(void) const { return rpid->GetDirection();}
			
}